
#include <Application.h>
#include <OS.h>
#include <time.h>

#include "DebugClient.h"


const int32 ALI_DEBUG_MSG = 'aldb';
const char *ALI_DEBUG_MONITOR_NAME = "application/x-vnd.ali-DebugMonitor";
DebugClient debug;

DebugClient::DebugClient() : server(ALI_DEBUG_MONITOR_NAME)
{}

void DebugClient::GetSignature(void)
{
	if (signature == BString())
	{
		app_info ai;
		be_app->GetAppInfo(&ai); 
		signature = ai.signature;
	}	
}

void DebugClient::GetBasicInfo(BMessage *message)
{
	message->AddInt32("Time", time(NULL));
	
	thread_id callingthreadid = find_thread(NULL);
	message->AddInt32("Thread", callingthreadid);

	thread_info tinfo;
	get_thread_info(callingthreadid, &tinfo);
	message->AddString("Threadname", tinfo.name);
}

void DebugClient::handletrace(const char *message, 
		const char *file,
		unsigned int line,
		const char *function)
{
	GetSignature();
		
	BMessage m(ALI_DEBUG_MSG);
	GetBasicInfo(&m);
		
	m.AddString("App", signature.String());	
	m.AddString("Type", "trace");
	m.AddString("Message", message);
	m.AddString("File", file);
	m.AddInt32("Line", line);
	m.AddString("Function", function);

	// m.PrintToStream();
	server.SendMessage(&m);
}
	
void DebugClient::handleassertfail(const char *assertion,
		const char *file,
		unsigned int line,
		const char *function)
{
	GetSignature();
	
	BMessage m(ALI_DEBUG_MSG);
	GetBasicInfo(&m);
				
	m.AddString("App", signature.String());	
	m.AddString("Type", "assertfail");
	m.AddString("Message", assertion);
	m.AddString("File", file);
	m.AddInt32("Line", line);
	m.AddString("Function", function);
	
	server.SendMessage(&m);	
		
	abort();
}		