#include "Tracer.h"

#include <TextView.h>
#include <Application.h>
#include <strstream.h>
#include <String.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <ScrollView.h>
#include <ScrollBar.h>

const unsigned MENU_CLEAR = 'mClr';

TracerWindow::TracerWindow(const BRect &frame, const char* name) 
	: BWindow(frame, name, B_DOCUMENT_WINDOW, 0)
{
	BMenuBar *pMenuBar = new BMenuBar(BRect(), "Menu Bar");
	BMenu* pMenu = new BMenu("Debug");
	BMenuItem* pClearItem = new BMenuItem("Clear",
		new BMessage(MENU_CLEAR));
	pClearItem->SetTarget(NULL, this);
	pMenu->AddItem(pClearItem);
	pMenu->AddSeparatorItem();

	BMenuItem* pAboutItem = new BMenuItem("About...",
		new BMessage(B_ABOUT_REQUESTED));
	pAboutItem->SetTarget(NULL, be_app);
	pMenu->AddItem(pAboutItem);

	pMenu->AddItem(new BMenuItem("Close",
		new BMessage(B_QUIT_REQUESTED), 'q'));
	pMenuBar->AddItem(pMenu);
	AddChild(pMenuBar);

	float menuHeight = pMenuBar->Bounds().Height();
	// set up a rectangle and instantiate a new view
	BRect r( Bounds() );
	r.Set(r.left, 
		r.top+menuHeight+1, 
		r.right-B_V_SCROLL_BAR_WIDTH, 
		r.bottom-B_H_SCROLL_BAR_HEIGHT);
	
	aView = new BListView(r, "tracer", B_SINGLE_SELECTION_LIST,
			B_FOLLOW_ALL_SIDES); 
   	AddChild(new BScrollView("scroll_tracer", aView, 
            B_FOLLOW_ALL_SIDES, 0, true, true));
}

bool TracerWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

const BString TracerWindow::FormatMessage(BMessage *message)
{
	ostrstream os;
	
	/* 
	os << message->FindString("Function") 
	   << "(" << message->FindString("File")
	   << ":" << message->FindInt32("Line")
	   << ") " << message->FindString("Message") << ends;
	*/
	
	os << message->FindString("Function") << ": " 
	   << message->FindString("Message") << ends;
	       
	BString s(os.str());
	return s;
}

void TracerWindow::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
	case 'aldb':
		//message->PrintToStream();
		aView->AddItem(new BStringItem(FormatMessage(message).String()));
		break;
	case MENU_CLEAR:
		while (!aView->IsEmpty()) 
       		delete aView->RemoveItem((int32)0);
		break;
	default:
		BWindow::MessageReceived(message);
		break;
	}
}

